/*
 * status-area-applet-mediaplayer
 *
 * Copyright (C) 2010 Faheem Pervez <trippin1@gmail.com>. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *      
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *      
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 */

/* status-area-applet-mediaplayer.vala */

using DBus;
using Mafw;
using GConf;
using Gtk;
using Hildon;

const string WANTED_RENDERER = "Mafw-Gst-Renderer";
const string WANTED_SOURCE = "Mafw-Tracker-Source";

class MediaPlayerStatusMenuItem : HD.StatusMenuItem
{
	private const string gettext_pkg = Config.GETTEXT_PACKAGE; /* Fuck you, Vala */
	private Hildon.Button button;
	private Gtk.IconTheme theme;
	private DBus.Connection conn;
	private dynamic DBus.Object mediaplayer;
	private unowned Mafw.Registry registry;
	private Mafw.Renderer gst_renderer;
	private Mafw.Source tracker_source;
	
	private void button_set_launch_labels_and_picture (bool label = true, bool @value = true, bool picture = true)
	{
		if (label)
			button.set_title(IntlPimped.dgettext("mediaplayer", "mp_ti_launch_first"));
		if (value)
			button.set_value(IntlPimped.dgettext("mediaplayer", "mp_ti_launch_second"));
		if (picture)
		{
			Gdk.Pixbuf pixbuf = theme.load_icon("mediaplayer_default_album", 64, IconLookupFlags.NO_SVG);
			pixbuf = pixbuf.scale_simple(48, 48, Gdk.InterpType.BILINEAR);
			Image image = new Image.from_pixbuf(pixbuf);
			button.set_image(image);
		}
	}
	
	private void button_set_song_labels_and_picture (string? label = null, string? @value = null, string? picuri = null)
	{
		button.set_title(label ?? IntlPimped.dgettext("mediaplayer", "mp_li_unknown_song"));
		button.set_value(value ?? IntlPimped.dgettext("mediaplayer", "mp_li_unknown_artist"));
		
		string? picture = picuri != null ? Filename.from_uri(picuri) : picuri; 

		if (picture == null || !FileUtils.test(picture, FileTest.EXISTS))
		{
			button_set_launch_labels_and_picture (false, false, true);
		}
		else
		{
			Gdk.Pixbuf pixbuf = new Gdk.Pixbuf.from_file_at_scale(picture, 48, 48, true);
			Image image = new Image.from_pixbuf(pixbuf);
			button.set_image(image);			
		}
	}
	
	private void renderer_on_media_changed (int index, string object_id)
		requires (tracker_source != null)
		requires (index >= 0)
	{
		SourceMetadataResultCb metadata_cb = (source, object_id, metadata) => {
			weak string artist = metadata_first(metadata, "artist").get_string ();
			weak string title = metadata_first(metadata, "title").get_string ();
			weak string album = metadata_first(metadata, "album-art-small-uri").get_string ();

			button_set_song_labels_and_picture (title, artist, album);
		};

		weak string metadata_keys[4];
		metadata_keys[0] = "artist";
		metadata_keys[1] = "title";
		metadata_keys[2] = "album-art-small-uri";
		metadata_keys[3] = null;

		tracker_source.get_metadata(object_id, metadata_keys, metadata_cb);
	}
	
	private void registry_on_renderer_added (Mafw.Renderer renderer)
	{
		RendererStatusCB renderer_status_cb = (self, playlist, index, state, object_id) => {
			renderer_on_media_changed ((int) index, object_id);
		};

		if (renderer.get_name() == WANTED_RENDERER)
		{
			gst_renderer = renderer;
			gst_renderer.media_changed += renderer_on_media_changed;
			gst_renderer.get_status (renderer_status_cb);
		}
	}

	private void registry_on_renderer_removed (Mafw.Renderer renderer)
	{
		if (renderer.get_name() == WANTED_RENDERER)
			gst_renderer = null;
	}
	
	private void registry_on_source_added (Mafw.Source source)
	{
		if (source.get_name() == WANTED_SOURCE)
			tracker_source = source;
	}

	private void registry_on_source_removed (Mafw.Source source)
	{
		if (source.get_name() == WANTED_SOURCE)
			tracker_source = null;
	}

	construct // Why can't I just add the following code to the [instance_]init function like I would in GObject C? :(
	{
		unowned GLib.List<Mafw.Renderer> rend_list;
		unowned GLib.List<Mafw.Source> src_list;
				
		Intl.setlocale (LocaleCategory.ALL, "");
		Intl.bindtextdomain (gettext_pkg, Config.LOCALEDIR);
		Intl.textdomain (gettext_pkg);
		
		theme = IconTheme.get_default();
		conn = DBus.Bus.get (DBus.BusType.SYSTEM);
		mediaplayer = conn.get_object ("com.nokia.mediaplayer", "/com/nokia/osso/mediaplayer", "com.nokia.mediaplayer");
		
		button = new Hildon.Button (SizeType.FINGER_HEIGHT | SizeType.AUTO_WIDTH, ButtonArrangement.VERTICAL);
		button.set_alignment (0.0f, 0.5f, 1.0f, 1.0f);
		button.set_style (Hildon.ButtonStyle.PICKER);
		
		button_set_launch_labels_and_picture ();
				
		button.clicked.connect (() => {
			mediaplayer.open_mp_now_playing (); /* I love you, Vala */
		});
		
		registry = registry.get_instance();
		shared_init (registry);
		
		registry.renderer_added += registry_on_renderer_added;
		registry.renderer_removed += registry_on_renderer_removed;
		registry.source_added += registry_on_source_added;
		registry.source_removed += registry_on_source_removed;
		
		rend_list = registry.get_renderers ();
		foreach (Mafw.Renderer renderer in rend_list)
			registry_on_renderer_added (renderer);
		
		src_list = registry.get_sources ();
		foreach (Mafw.Source source in src_list)
			registry_on_source_added (source);
		
		add (button);
		show_all();
	}
}


/** 
 * Vala code can't use the HD_DEFINE_PLUGIN_MODULE macro, but it handles
 * most of the class registration issues itself. Only this code from
 * HD_PLUGIN_MODULE_SYMBOLS_CODE has to be has to be included manually
 * to register with hildon-desktop:
 */
[ModuleInit]
public void hd_plugin_module_load (TypeModule plugin)
{
	// [ModuleInit] registers types automatically
	((HD.PluginModule) plugin).add_type (typeof (MediaPlayerStatusMenuItem));
}

public void hd_plugin_module_unload (HD.PluginModule plugin) {}
